from dateutil.relativedelta import relativedelta
from edc_lab_panel.panels import (
    blood_glucose_panel,
    fbc_panel,
    hba1c_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
)
from edc_visit_schedule.schedule import Schedule
from edc_visit_schedule.visit import (
    Crf,
    CrfCollection,
    Requisition,
    RequisitionCollection,
    Visit,
)
from edc_visit_schedule.visit_schedule import VisitSchedule

from edc_lab_results.tests.consents import consent_v1

crfs = CrfCollection(
    Crf(show_order=1, model="edc_lab_results.BloodResultsFbc", required=True),
)

requisitions = RequisitionCollection(
    Requisition(show_order=10, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=20, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=rft_panel, required=False, additional=False),
    Requisition(show_order=40, panel=lipids_panel, required=False, additional=False),
    Requisition(show_order=50, panel=blood_glucose_panel, required=False, additional=False),
    Requisition(show_order=60, panel=hba1c_panel, required=False, additional=False),
)


visit0 = Visit(
    code="1000",
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=requisitions,
    crfs=crfs,
    crfs_unscheduled=None,
    requisitions_unscheduled=None,
    facility_name="7-day-clinic",
)

visit1 = Visit(
    code="2000",
    title="Day 2",
    timepoint=1,
    rbase=relativedelta(days=1),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=requisitions,
    crfs=crfs,
    crfs_unscheduled=None,
    requisitions_unscheduled=None,
    facility_name="7-day-clinic",
)


schedule = Schedule(
    name="schedule",
    onschedule_model="edc_metadata.onschedule",
    offschedule_model="edc_metadata.offschedule",
    consent_definitions=[consent_v1],
    appointment_model="edc_appointment.appointment",
)

schedule.add_visit(visit0)
schedule.add_visit(visit1)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="edc_metadata.deathreport",
)

visit_schedule.add_schedule(schedule)



from django.db import models
from edc_lab_panel.model_mixin_factory import reportable_result_model_mixin_factory
from edc_reportable.units import PLUS


class ProteinuriaModelMixin(
    reportable_result_model_mixin_factory(
        utest_id="proteinuria",
        verbose_name="Proteinuria",
        units_choices=((PLUS, PLUS),),
        decimal_places=1,
    ),
    models.Model,
):
    class Meta:
        abstract = True



from typing import Any

from django import forms
from edc_constants.constants import FASTING, YES
from edc_glucose.utils import validate_glucose_as_millimoles_per_liter


class BloodResultsFbgFormValidatorMixin:
    @property
    def reportables_evaluator_options(self: Any):
        if not self.cleaned_data.get("fasting"):
            raise forms.ValidationError({"fasting": "This field is required."})
        fasting = (
            True
            if (
                (self.cleaned_data.get("fasting") == FASTING)
                or (self.cleaned_data.get("fasting") == YES)
            )
            else False
        )
        return dict(fasting=fasting)

    def evaluate_value(self: Any, field_name: str):
        if field_name == "ifg_value":
            validate_glucose_as_millimoles_per_liter("ifg", self.cleaned_data)



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_crf.model_mixins import CrfModelMixin, CrfWithActionModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.model_mixin_factory import reportable_result_model_mixin_factory
from edc_lab_panel.panels import fbc_panel
from edc_model import models as edc_models
from edc_reportable import GRAMS_PER_DECILITER

from edc_lab_results import BLOOD_RESULTS_FBC_ACTION
from edc_lab_results.model_mixins import (
    BloodResultsModelMixin,
    HaemoglobinModelMixin,
    Hba1cModelMixin,
    HctModelMixin,
    PlateletsModelMixin,
    RbcModelMixin,
    WbcModelMixin,
)


class BloodResultsFbc(
    CrfWithActionModelMixin,
    CrfWithRequisitionModelMixin,
    HaemoglobinModelMixin,
    HctModelMixin,
    RbcModelMixin,
    WbcModelMixin,
    PlateletsModelMixin,
    BloodResultsModelMixin,
    edc_models.BaseUuidModel,
):
    action_name = BLOOD_RESULTS_FBC_ACTION

    tracking_identifier_prefix = "FB"

    lab_panel = fbc_panel

    class Meta(CrfWithActionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Blood Result: FBC"
        verbose_name_plural = "Blood Results: FBC"


class AbcModelMixin(
    reportable_result_model_mixin_factory(
        "hct", ((GRAMS_PER_DECILITER, GRAMS_PER_DECILITER),)
    ),
    models.Model,
):
    # HCT
    hct_value = models.DecimalField(
        validators=[MinValueValidator(1.00), MaxValueValidator(999.00)],
        verbose_name="Hematocrit",
        decimal_places=2,
        max_digits=6,
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True


# this model does not include the requisition and action item mixins
class BloodResultsHba1c(
    CrfModelMixin,
    CrfWithRequisitionModelMixin,
    Hba1cModelMixin,
    BloodResultsModelMixin,
    edc_models.BaseUuidModel,
):
    class Meta(edc_models.BaseUuidModel.Meta):
        verbose_name = "HbA1c"
        verbose_name_plural = "HbA1c"
